@@ -1,41 +1,23 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 #import "GPBExtensionInternals.h"
 
 #import <objc/runtime.h>
 
+#import "GPBCodedInputStream.h"
 #import "GPBCodedInputStream_PackagePrivate.h"
+#import "GPBCodedOutputStream.h"
 #import "GPBCodedOutputStream_PackagePrivate.h"
+#import "GPBDescriptor.h"
 #import "GPBDescriptor_PackagePrivate.h"
+#import "GPBMessage.h"
 #import "GPBMessage_PackagePrivate.h"
+#import "GPBUtilities.h"
 #import "GPBUtilities_PackagePrivate.h"
 
 GPB_INLINE size_t DataTypeSize(GPBDataType dataType) {
@@ -115,12 +97,7 @@ static size_t ComputeSerializedSizeIncludingTagOfObject(GPBExtensionDescription
     FIELD_CASE2(Bytes)
     FIELD_CASE2(String)
     FIELD_CASE2(Group)
-    case GPBDataTypeMessage:
-      if (GPBExtensionIsWireFormat(description)) {
-        return GPBComputeMessageSetExtensionSize(description->fieldNumber, object);
-      } else {
-        return GPBComputeMessageSize(description->fieldNumber, object);
-      }
+    FIELD_CASE2(Message)
   }
 #undef FIELD_CASE
 #undef FIELD_CASE2
@@ -178,13 +155,7 @@ static void WriteObjectIncludingTagToCodedOutputStream(id object,
     FIELD_CASE2(Bytes)
     FIELD_CASE2(String)
     FIELD_CASE2(Group)
-    case GPBDataTypeMessage:
-      if (GPBExtensionIsWireFormat(description)) {
-        [output writeMessageSetExtension:description->fieldNumber value:object];
-      } else {
-        [output writeMessage:description->fieldNumber value:object];
-      }
-      return;
+    FIELD_CASE2(Message)
   }
 #undef FIELD_CASE
 #undef FIELD_CASE2
