@@ -1,32 +1,9 @@
 # Protocol Buffers - Google's data interchange format
 # Copyright 2008 Google Inc.  All rights reserved.
-# https://developers.google.com/protocol-buffers/
 #
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are
-# met:
-#
-#     * Redistributions of source code must retain the above copyright
-# notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above
-# copyright notice, this list of conditions and the following disclaimer
-# in the documentation and/or other materials provided with the
-# distribution.
-#     * Neither the name of Google Inc. nor the names of its
-# contributors may be used to endorse or promote products derived from
-# this software without specific prior written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# Use of this source code is governed by a BSD-style
+# license that can be found in the LICENSE file or at
+# https://developers.google.com/open-source/licenses/bsd
 
 """Tests for google.protobuf.descriptor_database."""
 
@@ -35,12 +12,12 @@ __author__ = 'matthewtoia@google.com (Matt Toia)'
 import unittest
 import warnings
 
-from google.protobuf import unittest_pb2
 from google.protobuf import descriptor_pb2
 from google.protobuf.internal import factory_test2_pb2
 from google.protobuf.internal import no_package_pb2
 from google.protobuf.internal import testing_refleaks
 from google.protobuf import descriptor_database
+from google.protobuf import unittest_pb2
 
 
 @testing_refleaks.TestCase
@@ -79,8 +56,26 @@ class DescriptorDatabaseTest(unittest.TestCase):
         'google.protobuf.python.internal.Factory2Enum.FACTORY_2_VALUE_0'))
     self.assertEqual(file_desc_proto, db.FindFileContainingSymbol(
         'google.protobuf.python.internal.FACTORY_2_VALUE_0'))
-    self.assertEqual(file_desc_proto2, db.FindFileContainingSymbol(
-        '.NO_PACKAGE_VALUE_0'))
+    self.assertEqual(
+        file_desc_proto2, db.FindFileContainingSymbol('NO_PACKAGE_VALUE_0')
+    )
+    self.assertEqual(
+        file_desc_proto2, db.FindFileContainingSymbol('.NO_PACKAGE_VALUE_0')
+    )
+    self.assertEqual(
+        file_desc_proto2, db.FindFileContainingSymbol('NoPackageMessage')
+    )
+    self.assertEqual(
+        file_desc_proto2, db.FindFileContainingSymbol('.NoPackageMessage')
+    )
+    self.assertEqual(
+        file_desc_proto2,
+        db.FindFileContainingSymbol('NoPackageEnum'),
+    )
+    self.assertEqual(
+        file_desc_proto2,
+        db.FindFileContainingSymbol('.NoPackageEnum'),
+    )
     # Can find top level extension.
     self.assertEqual(file_desc_proto, db.FindFileContainingSymbol(
         'google.protobuf.python.internal.another_field'))
@@ -93,15 +88,20 @@ class DescriptorDatabaseTest(unittest.TestCase):
         unittest_pb2.DESCRIPTOR.serialized_pb)
     db.Add(file_desc_proto2)
     self.assertEqual(file_desc_proto2, db.FindFileContainingSymbol(
-        'protobuf_unittest.TestService'))
+        'proto2_unittest.TestService'))
 
     # Non-existent field under a valid top level symbol can also be
     # found. The behavior is the same with protobuf C++.
     self.assertEqual(file_desc_proto2, db.FindFileContainingSymbol(
-        'protobuf_unittest.TestAllTypes.none_field'))
+        'proto2_unittest.TestAllTypes.none_field'))
 
-    with self.assertRaisesRegex(KeyError, r'\'protobuf_unittest\.NoneMessage\''):
-      db.FindFileContainingSymbol('protobuf_unittest.NoneMessage')
+    with self.assertRaisesRegex(KeyError, r'\'proto2_unittest\.NoneMessage\''):
+      db.FindFileContainingSymbol('proto2_unittest.NoneMessage')
+
+    with self.assertRaises(KeyError):
+      db.FindFileContainingSymbol(
+          '.google.protobuf.python.internal.FACTORY_2_VALUE_0'
+      )
 
   def testConflictRegister(self):
     db = descriptor_database.DescriptorDatabase()
@@ -119,9 +119,9 @@ class DescriptorDatabaseTest(unittest.TestCase):
       self.assertIs(w[0].category, RuntimeWarning)
       self.assertIn('Conflict register for file "other_file2": ',
                     str(w[0].message))
-      self.assertIn('already defined in file '
-                    '"google/protobuf/unittest.proto"',
-                    str(w[0].message))
+      self.assertIn(
+          'already defined in file '
+          '"google/protobuf/unittest.proto"', str(w[0].message))
 
 if __name__ == '__main__':
   unittest.main()
