@@ -36,24 +36,6 @@ class RuntimeVersionTest(unittest.TestCase):
           gencode_domain, 1, 2, 3, '', 'foo.proto'
       )
 
-  def test_mismatched_major_disallowed(self):
-    gencode_version_string = f'{1}.{runtime_version.MINOR}.{runtime_version.PATCH}{runtime_version.SUFFIX}'
-    runtime_version_string = f'{runtime_version.MAJOR}.{runtime_version.MINOR}.{runtime_version.PATCH}{runtime_version.SUFFIX}'
-    with self.assertRaisesRegex(
-        runtime_version.VersionError,
-        'Detected mismatched Protobuf Gencode/Runtime major versions when'
-        f' loading foo.proto: gencode {gencode_version_string} runtime'
-        f' {runtime_version_string}',
-    ):
-      runtime_version.ValidateProtobufRuntimeVersion(
-          runtime_version.DOMAIN,
-          1,
-          runtime_version.MINOR,
-          runtime_version.PATCH,
-          runtime_version.SUFFIX,
-          'foo.proto',
-      )
-
   def test_same_version_allowed(self):
     runtime_version.ValidateProtobufRuntimeVersion(
         runtime_version.DOMAIN,
@@ -103,34 +85,6 @@ class RuntimeVersionTest(unittest.TestCase):
           'foo.proto',
       )
 
-  def test_different_suffix_disallowed(self):
-    with self.assertRaisesRegex(
-        runtime_version.VersionError,
-        'Detected mismatched Protobuf Gencode/Runtime version suffixes when'
-        ' loading foo.proto',
-    ):
-      runtime_version.ValidateProtobufRuntimeVersion(
-          runtime_version.DOMAIN,
-          runtime_version.MAJOR,
-          runtime_version.MINOR,
-          runtime_version.PATCH,
-          '-noflag',
-          'foo.proto',
-      )
-
-  def test_gencode_one_major_version_older_warning(self):
-    with self.assertWarnsRegex(
-        Warning, expected_regex='is exactly one major version older'
-    ):
-      runtime_version.ValidateProtobufRuntimeVersion(
-          runtime_version.DOMAIN,
-          runtime_version.MAJOR - 1,
-          runtime_version.MINOR,
-          runtime_version.PATCH,
-          runtime_version.SUFFIX,
-          'foo.proto',
-      )
-
 
 if __name__ == '__main__':
   unittest.main()
