@@ -7,16 +7,27 @@
 
 //! Tests covering accessors for singular bool, int32, int64, and bytes fields.
 
-use googletest::prelude::*;
-use protobuf::{MutProxy, Optional};
-use unittest_proto::{test_all_types, TestAllTypes};
+#[cfg(not(bzl))]
+mod protos;
+#[cfg(not(bzl))]
+use protos::*;
 
-#[test]
+use googletest::prelude::*;
+use protobuf::prelude::*;
+
+use protobuf::{Optional, ProtoBytes, ProtoStr, ProtoString};
+use std::borrow::Cow;
+use std::ffi::OsString;
+use std::rc::Rc;
+use std::sync::Arc;
+use unittest_rust_proto::{test_all_types, TestAllTypes};
+
+#[gtest]
 fn test_default_accessors() {
     let msg: TestAllTypes = Default::default();
     assert_that!(
         msg,
-        matches_pattern!(TestAllTypes{
+        matches_pattern!(&TestAllTypes{
             default_int32(): eq(41),
             default_int64(): eq(42),
             default_uint32(): eq(43),
@@ -36,7 +47,7 @@ fn test_default_accessors() {
     assert_that!(msg.default_bytes(), eq("world".as_bytes()));
 }
 
-#[test]
+#[gtest]
 fn test_optional_fixed32_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.has_optional_fixed32(), eq(false));
@@ -54,7 +65,7 @@ fn test_optional_fixed32_accessors() {
     assert_that!(msg.optional_fixed32(), eq(0));
 }
 
-#[test]
+#[gtest]
 fn test_default_fixed32_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_fixed32(), eq(47));
@@ -72,7 +83,7 @@ fn test_default_fixed32_accessors() {
     assert_that!(msg.default_fixed32_opt(), eq(Optional::Unset(47)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_fixed64_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_fixed64_opt(), eq(Optional::Unset(0)));
@@ -91,7 +102,7 @@ fn test_optional_fixed64_accessors() {
     assert_that!(msg.optional_fixed64(), eq(0));
 }
 
-#[test]
+#[gtest]
 fn test_default_fixed64_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_fixed64(), eq(48));
@@ -110,7 +121,7 @@ fn test_default_fixed64_accessors() {
     assert_that!(msg.default_fixed64_opt(), eq(Optional::Unset(48)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_int32_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_int32_opt(), eq(Optional::Unset(0)));
@@ -129,7 +140,7 @@ fn test_optional_int32_accessors() {
     assert_that!(msg.optional_int32(), eq(0));
 }
 
-#[test]
+#[gtest]
 fn test_default_int32_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_int32(), eq(41));
@@ -152,7 +163,7 @@ fn test_default_int32_accessors() {
     assert_that!(msg.default_int32_opt(), eq(Optional::Unset(41)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_int64_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_int64_opt(), eq(Optional::Unset(0)));
@@ -167,7 +178,7 @@ fn test_optional_int64_accessors() {
     assert_that!(msg.optional_int64(), eq(0));
 }
 
-#[test]
+#[gtest]
 fn test_default_int64_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_int64(), eq(42));
@@ -182,7 +193,7 @@ fn test_default_int64_accessors() {
     assert_that!(msg.default_int64_opt(), eq(Optional::Unset(42)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_sint32_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_sint32_opt(), eq(Optional::Unset(0)));
@@ -197,7 +208,7 @@ fn test_optional_sint32_accessors() {
     assert_that!(msg.optional_sint32(), eq(0));
 }
 
-#[test]
+#[gtest]
 fn test_default_sint32_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_sint32(), eq(-45));
@@ -212,7 +223,7 @@ fn test_default_sint32_accessors() {
     assert_that!(msg.default_sint32_opt(), eq(Optional::Unset(-45)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_sint64_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_sint64_opt(), eq(Optional::Unset(0)));
@@ -227,7 +238,7 @@ fn test_optional_sint64_accessors() {
     assert_that!(msg.optional_sint64(), eq(0));
 }
 
-#[test]
+#[gtest]
 fn test_default_sint64_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_sint64(), eq(46));
@@ -242,7 +253,7 @@ fn test_default_sint64_accessors() {
     assert_that!(msg.default_sint64_opt(), eq(Optional::Unset(46)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_uint32_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_uint32_opt(), eq(Optional::Unset(0)));
@@ -257,7 +268,7 @@ fn test_optional_uint32_accessors() {
     assert_that!(msg.optional_uint32(), eq(0));
 }
 
-#[test]
+#[gtest]
 fn test_default_uint32_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_uint32(), eq(43));
@@ -272,7 +283,7 @@ fn test_default_uint32_accessors() {
     assert_that!(msg.default_uint32_opt(), eq(Optional::Unset(43)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_uint64_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_uint64_opt(), eq(Optional::Unset(0)));
@@ -287,7 +298,7 @@ fn test_optional_uint64_accessors() {
     assert_that!(msg.optional_uint64(), eq(0));
 }
 
-#[test]
+#[gtest]
 fn test_default_uint64_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_uint64(), eq(44));
@@ -302,7 +313,7 @@ fn test_default_uint64_accessors() {
     assert_that!(msg.default_uint64_opt(), eq(Optional::Unset(44)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_float_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_float_opt(), eq(Optional::Unset(0.0)));
@@ -317,7 +328,7 @@ fn test_optional_float_accessors() {
     assert_that!(msg.optional_float(), eq(0.0));
 }
 
-#[test]
+#[gtest]
 fn test_default_float_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_float(), eq(51.5));
@@ -332,7 +343,7 @@ fn test_default_float_accessors() {
     assert_that!(msg.default_float_opt(), eq(Optional::Unset(51.5)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_double_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_double_opt(), eq(Optional::Unset(0.0)));
@@ -347,7 +358,7 @@ fn test_optional_double_accessors() {
     assert_that!(msg.optional_double(), eq(0.0));
 }
 
-#[test]
+#[gtest]
 fn test_default_double_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_double(), eq(52e3));
@@ -362,7 +373,7 @@ fn test_default_double_accessors() {
     assert_that!(msg.default_double_opt(), eq(Optional::Unset(52e3)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_bool_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_bool_opt(), eq(Optional::Unset(false)));
@@ -374,7 +385,7 @@ fn test_optional_bool_accessors() {
     assert_that!(msg.optional_bool_opt(), eq(Optional::Unset(false)));
 }
 
-#[test]
+#[gtest]
 fn test_default_bool_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_bool(), eq(true));
@@ -389,10 +400,10 @@ fn test_default_bool_accessors() {
     assert_that!(msg.default_bool_opt(), eq(Optional::Unset(true)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_bytes_accessors() {
     let mut msg = TestAllTypes::new();
-    assert_that!(*msg.optional_bytes(), empty());
+    assert_that!(*msg.optional_bytes(), is_empty());
     assert_that!(msg.has_optional_bytes(), eq(false));
     assert_that!(msg.optional_bytes_opt(), eq(Optional::Unset(&b""[..])));
 
@@ -405,17 +416,59 @@ fn test_optional_bytes_accessors() {
     assert_that!(msg.optional_bytes_opt(), eq(Optional::Set(&b"hello world"[..])));
 
     msg.clear_optional_bytes();
-    assert_that!(*msg.optional_bytes(), empty());
+    assert_that!(*msg.optional_bytes(), is_empty());
     assert_that!(msg.has_optional_bytes(), eq(false));
     assert_that!(msg.optional_bytes_opt(), eq(Optional::Unset(&b""[..])));
 
     msg.set_optional_bytes(b"");
-    assert_that!(*msg.optional_bytes(), empty());
+    assert_that!(*msg.optional_bytes(), is_empty());
     assert_that!(msg.has_optional_bytes(), eq(true));
     assert_that!(msg.optional_bytes_opt(), eq(Optional::Set(&b""[..])));
 }
 
-#[test]
+#[gtest]
+fn test_into_proxied_for_bytes() {
+    let mut msg = TestAllTypes::new();
+
+    // &[u8]
+    let bytes: &[u8] = b"first";
+    msg.set_optional_bytes(bytes);
+    assert_that!(msg.optional_bytes(), eq(bytes));
+
+    // &[u8; N]
+    msg.set_optional_bytes(b"second");
+    assert_that!(msg.optional_bytes(), eq(b"second"));
+
+    // Vec<u8>
+    msg.set_optional_bytes(Vec::from(b"third"));
+    assert_that!(msg.optional_bytes(), eq(b"third"));
+
+    // ProtoBytes
+    msg.set_optional_bytes(ProtoBytes::from(b"fourth"));
+    assert_that!(msg.optional_bytes(), eq(b"fourth"));
+
+    // Box<[u8]>
+    msg.set_optional_bytes(Box::from(b"fifth".to_owned()));
+    assert_that!(msg.optional_bytes(), eq(b"fifth"));
+
+    // Cow<[u8]>
+    msg.set_optional_bytes(Cow::from(b"sixth"));
+    assert_that!(msg.optional_bytes(), eq(b"sixth"));
+
+    // Rc<[u8]>
+    msg.set_optional_bytes(Rc::from(b"seventh".to_owned()));
+    assert_that!(msg.optional_bytes(), eq(b"seventh"));
+
+    // Arc<[u8]>
+    msg.set_optional_bytes(Arc::from(b"eighth".to_owned()));
+    assert_that!(msg.optional_bytes(), eq(b"eighth"));
+
+    // &Vec<u8>
+    msg.set_optional_bytes(&Vec::from(b"ninth"));
+    assert_that!(msg.optional_bytes(), eq(b"ninth"));
+}
+
+#[gtest]
 fn test_nonempty_default_bytes_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_bytes(), eq(b"world"));
@@ -436,7 +489,7 @@ fn test_nonempty_default_bytes_accessors() {
     assert_that!(msg.default_bytes_opt(), eq(Optional::Unset(&b"world"[..])));
 
     msg.set_default_bytes(b"");
-    assert_that!(*msg.default_bytes(), empty());
+    assert_that!(*msg.default_bytes(), is_empty());
     assert_that!(msg.default_bytes_opt(), eq(Optional::Set(&b""[..])));
 
     msg.clear_default_bytes();
@@ -444,7 +497,7 @@ fn test_nonempty_default_bytes_accessors() {
     assert_that!(msg.default_bytes_opt(), eq(Optional::Unset(&b"world"[..])));
 }
 
-#[test]
+#[gtest]
 fn test_optional_string_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_string(), eq(""));
@@ -470,7 +523,56 @@ fn test_optional_string_accessors() {
     assert_that!(msg.optional_string_opt(), eq(Optional::Unset("".into())));
 }
 
-#[test]
+#[gtest]
+fn test_into_proxied_for_string() {
+    let mut msg = TestAllTypes::new();
+
+    // &str
+    msg.set_optional_string("first");
+    assert_that!(msg.optional_string(), eq("first"));
+
+    // String
+    msg.set_optional_string("second".to_string());
+    assert_that!(msg.optional_string(), eq("second"));
+
+    // ProtoStr
+    msg.set_optional_string(ProtoStr::from_str("third"));
+    assert_that!(msg.optional_string(), eq("third"));
+
+    // ProtoString
+    msg.set_optional_string(ProtoString::from("fourth"));
+    assert_that!(msg.optional_string(), eq("fourth"));
+
+    // OsString
+    msg.set_optional_string(OsString::from("fifth"));
+    assert_that!(msg.optional_string(), eq("fifth"));
+
+    // OsStr
+    msg.set_optional_string(OsString::from("sixth").as_os_str());
+    assert_that!(msg.optional_string(), eq("sixth"));
+
+    // Box<str>
+    msg.set_optional_string(Box::from("seventh"));
+    assert_that!(msg.optional_string(), eq("seventh"));
+
+    // Cow<str>
+    msg.set_optional_string(Cow::from("eighth"));
+    assert_that!(msg.optional_string(), eq("eighth"));
+
+    // Rc<str>
+    msg.set_optional_string(Rc::from("ninth"));
+    assert_that!(msg.optional_string(), eq("ninth"));
+
+    // Arc<str>
+    msg.set_optional_string(Arc::from("tenth"));
+    assert_that!(msg.optional_string(), eq("tenth"));
+
+    // &String
+    msg.set_optional_string(&"eleventh".to_string());
+    assert_that!(msg.optional_string(), eq("eleventh"));
+}
+
+#[gtest]
 fn test_nonempty_default_string_accessors() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_string(), eq("hello"));
@@ -496,42 +598,31 @@ fn test_nonempty_default_string_accessors() {
     assert_that!(msg.default_string_opt(), eq(Optional::Unset("hello".into())));
 }
 
-#[test]
+#[gtest]
 fn test_singular_msg_field() {
-    use test_all_types::*;
-
     let mut msg = TestAllTypes::new();
     let msg_view = msg.optional_nested_message();
     // testing reading an int inside a view
     assert_that!(msg_view.bb(), eq(0));
 
     assert_that!(msg.has_optional_nested_message(), eq(false));
-    let mut nested_msg_mut = msg.optional_nested_message_mut();
-
+    let nested_msg_mut = msg.optional_nested_message_mut();
     // test reading an int inside a mut
     assert_that!(nested_msg_mut.bb(), eq(0));
 
-    // Test setting an owned NestedMessage onto another message.
-    let mut new_nested = NestedMessage::new();
-    new_nested.set_bb(7);
-    nested_msg_mut.set(new_nested);
-    assert_that!(nested_msg_mut.bb(), eq(7));
-
     assert_that!(msg.has_optional_nested_message(), eq(true));
 }
 
-#[test]
+#[gtest]
 fn test_message_opt() {
     let msg = TestAllTypes::new();
-    let opt: Optional<
-        unittest_proto::test_all_types::NestedMessageView<'_>,
-        unittest_proto::test_all_types::NestedMessageView<'_>,
-    > = msg.optional_nested_message_opt();
+    let opt: Optional<unittest_rust_proto::test_all_types::NestedMessageView<'_>> =
+        msg.optional_nested_message_opt();
     assert_that!(opt.is_set(), eq(false));
     assert_that!(opt.into_inner().bb(), eq(0));
 }
 
-#[test]
+#[gtest]
 fn test_message_opt_set() {
     let mut msg = TestAllTypes::new();
     let submsg = test_all_types::NestedMessage::new();
@@ -540,7 +631,7 @@ fn test_message_opt_set() {
     assert_that!(msg.optional_nested_message_opt().is_set(), eq(false));
 }
 
-#[test]
+#[gtest]
 fn test_setting_submsg() {
     let mut msg = TestAllTypes::new();
     let submsg = test_all_types::NestedMessage::new();
@@ -558,7 +649,7 @@ fn test_setting_submsg() {
     assert_that!(msg.optional_nested_message_opt().is_set(), eq(false));
 }
 
-#[test]
+#[gtest]
 fn test_msg_mut_initializes() {
     let mut msg = TestAllTypes::new();
     assert_that!(msg.has_optional_nested_message(), eq(false));
@@ -574,7 +665,7 @@ fn test_msg_mut_initializes() {
     assert_that!(msg.optional_nested_message_opt().is_set(), eq(false));
 }
 
-#[test]
+#[gtest]
 fn test_optional_nested_enum_accessors() {
     use test_all_types::NestedEnum;
 
@@ -594,7 +685,7 @@ fn test_optional_nested_enum_accessors() {
     assert_that!(msg.optional_nested_enum(), eq(NestedEnum::Foo));
 }
 
-#[test]
+#[gtest]
 fn test_default_nested_enum_accessors() {
     use test_all_types::NestedEnum;
 
@@ -611,9 +702,9 @@ fn test_default_nested_enum_accessors() {
     assert_that!(msg.default_nested_enum_opt(), eq(Optional::Unset(NestedEnum::Bar)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_foreign_enum_accessors() {
-    use unittest_proto::ForeignEnum;
+    use unittest_rust_proto::ForeignEnum;
 
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_foreign_enum_opt(), eq(Optional::Unset(ForeignEnum::ForeignFoo)));
@@ -628,9 +719,9 @@ fn test_optional_foreign_enum_accessors() {
     assert_that!(msg.optional_foreign_enum(), eq(ForeignEnum::ForeignFoo));
 }
 
-#[test]
+#[gtest]
 fn test_default_foreign_enum_accessors() {
-    use unittest_proto::ForeignEnum;
+    use unittest_rust_proto::ForeignEnum;
 
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_foreign_enum(), eq(ForeignEnum::ForeignBar));
@@ -645,9 +736,9 @@ fn test_default_foreign_enum_accessors() {
     assert_that!(msg.default_foreign_enum_opt(), eq(Optional::Unset(ForeignEnum::ForeignBar)));
 }
 
-#[test]
+#[gtest]
 fn test_optional_import_enum_accessors() {
-    use unittest_proto::ImportEnum;
+    use unittest_import_rust_proto::ImportEnum;
 
     let mut msg = TestAllTypes::new();
     assert_that!(msg.optional_import_enum_opt(), eq(Optional::Unset(ImportEnum::ImportFoo)));
@@ -662,9 +753,9 @@ fn test_optional_import_enum_accessors() {
     assert_that!(msg.optional_import_enum(), eq(ImportEnum::ImportFoo));
 }
 
-#[test]
+#[gtest]
 fn test_default_import_enum_accessors() {
-    use unittest_proto::ImportEnum;
+    use unittest_import_rust_proto::ImportEnum;
 
     let mut msg = TestAllTypes::new();
     assert_that!(msg.default_import_enum(), eq(ImportEnum::ImportBar));
@@ -679,10 +770,10 @@ fn test_default_import_enum_accessors() {
     assert_that!(msg.default_import_enum_opt(), eq(Optional::Unset(ImportEnum::ImportBar)));
 }
 
-#[test]
+#[gtest]
 fn test_oneof_accessors() {
-    use unittest_proto::test_oneof2::{Foo::*, FooCase, NestedEnum};
-    use unittest_proto::TestOneof2;
+    use unittest_rust_proto::test_oneof2::{FooCase, FooOneof::*, NestedEnum};
+    use unittest_rust_proto::TestOneof2;
 
     let mut msg = TestOneof2::new();
     assert_that!(msg.foo(), matches_pattern!(not_set(_)));
@@ -730,11 +821,11 @@ fn test_oneof_accessors() {
     // TODO: Add tests covering a message-type field in a oneof.
 }
 
-#[test]
+#[gtest]
 fn test_msg_oneof_default_accessors() {
-    use unittest_proto::test_oneof2::{Bar::*, BarCase, NestedEnum};
+    use unittest_rust_proto::test_oneof2::{BarCase, BarOneof::*, NestedEnum};
 
-    let mut msg = unittest_proto::TestOneof2::new();
+    let mut msg = unittest_rust_proto::TestOneof2::new();
     assert_that!(msg.bar(), matches_pattern!(not_set(_)));
 
     msg.set_bar_int(7);
@@ -762,19 +853,7 @@ fn test_msg_oneof_default_accessors() {
     // TODO: Add tests covering a message-type field in a oneof.
 }
 
-#[test]
-fn test_set_message_from_view() {
-    use protobuf::MutProxy;
-
-    let mut m1 = TestAllTypes::new();
-    m1.set_optional_int32(1);
-    let mut m2 = TestAllTypes::new();
-    m2.as_mut().set(m1.as_view());
-
-    assert_that!(m2.optional_int32(), eq(1i32));
-}
-
-#[test]
+#[gtest]
 fn test_group() {
     let mut m = TestAllTypes::new();
 
@@ -787,7 +866,7 @@ fn test_group() {
     assert_that!(m.optionalgroup().a(), eq(7));
 }
 
-#[test]
+#[gtest]
 fn test_submsg_setter() {
     use test_all_types::*;
 
@@ -799,3 +878,75 @@ fn test_submsg_setter() {
 
     assert_that!(parent.optional_nested_message().bb(), eq(7));
 }
+
+#[gtest]
+fn test_clone() {
+    let mut m = TestAllTypes::new();
+    m.set_optional_int32(42);
+    let clone = m.clone();
+    assert_that!(clone.optional_int32(), eq(42));
+    m.clear_optional_int32();
+    assert_that!(m.has_optional_int32(), eq(false));
+    assert_that!(clone.has_optional_int32(), eq(true));
+    assert_that!(clone.optional_int32(), eq(42));
+}
+
+#[gtest]
+fn test_to_owned() {
+    let mut m = TestAllTypes::new();
+    m.set_optional_int32(42);
+    let clone = m.as_view().to_owned();
+    assert_that!(clone.optional_int32(), eq(42));
+
+    // to_owned should create a new message (modifying the original shouldn't affect
+    // the to_owned).
+    m.clear_optional_int32();
+    assert_that!(m.has_optional_int32(), eq(false));
+    assert_that!(clone.has_optional_int32(), eq(true));
+    assert_that!(clone.optional_int32(), eq(42));
+
+    let mut submsg_mut = m.optional_nested_message_mut();
+    submsg_mut.set_bb(7);
+    let submsg_clone = submsg_mut.to_owned();
+    assert_that!(submsg_clone.bb(), eq(7));
+    assert_that!(submsg_mut.bb(), eq(7));
+    submsg_mut.set_bb(8);
+    assert_that!(submsg_clone.bb(), eq(7));
+    assert_that!(submsg_mut.bb(), eq(8));
+}
+
+#[gtest]
+fn test_ctype_stringpiece() {
+    let mut msg = TestAllTypes::new();
+    assert_that!(msg.optional_string_piece(), eq(""));
+    assert_that!(msg.has_optional_string_piece(), eq(false));
+    msg.set_optional_string_piece("hello");
+    assert_that!(msg.optional_string_piece(), eq("hello"));
+    assert_that!(msg.has_optional_string_piece(), eq(true));
+}
+
+#[gtest]
+fn test_msg_clear() {
+    let mut m = TestAllTypes::new();
+    m.set_optional_int32(42);
+    assert_that!(m.has_optional_int32(), eq(true));
+    m.clear();
+    assert_that!(m.has_optional_int32(), eq(false));
+}
+
+#[gtest]
+fn test_submsg_clear() {
+    let mut m = TestAllTypes::new();
+    let mut sub = m.optional_nested_message_mut();
+    sub.set_bb(7);
+
+    assert_that!(m.has_optional_nested_message(), eq(true));
+    assert_that!(m.optional_nested_message().bb(), eq(7));
+
+    m.optional_nested_message_mut().clear();
+
+    // .clear() on the submsg doesn't affect its presence on the parent:
+    assert_that!(m.has_optional_nested_message(), eq(true));
+    // ...but it does clear the submsg's value:
+    assert_that!(m.optional_nested_message().bb(), eq(0));
+}
