@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // Author: kenton@google.com (Kenton Varda)
 //  Based on original Protocol Buffers design by
@@ -39,13 +16,16 @@
 #define GOOGLE_PROTOBUF_DYNAMIC_MESSAGE_H__
 
 #include <algorithm>
-#include <memory>
+#include <cstddef>
+#include <cstdint>
+#include <string>
 #include <vector>
 
 #include "absl/container/flat_hash_map.h"
+#include "absl/log/absl_log.h"
 #include "absl/synchronization/mutex.h"
+#include "google/protobuf/descriptor.h"
 #include "google/protobuf/message.h"
-#include "google/protobuf/port.h"
 #include "google/protobuf/reflection.h"
 #include "google/protobuf/repeated_field.h"
 
@@ -96,7 +76,10 @@ class PROTOBUF_EXPORT DynamicMessageFactory : public MessageFactory {
   //   parser to look for extensions in an alternate pool.  However, note that
   //   this is almost never what you want to do.  Almost all users should use
   //   the zero-arg constructor.
-  DynamicMessageFactory(const DescriptorPool* pool);
+#ifndef PROTOBUF_FUTURE_BREAKING_CHANGES
+  explicit
+#endif
+      DynamicMessageFactory(const DescriptorPool* PROTOBUF_NONNULL pool);
   DynamicMessageFactory(const DynamicMessageFactory&) = delete;
   DynamicMessageFactory& operator=(const DynamicMessageFactory&) = delete;
 
@@ -125,14 +108,15 @@ class PROTOBUF_EXPORT DynamicMessageFactory : public MessageFactory {
   // prototype, so these must be destroyed before the DynamicMessageFactory
   // is destroyed.
   //
-  // The given descriptor must outlive the returned message, and hence must
-  // outlive the DynamicMessageFactory.
+  // The given descriptor must be non-null and outlive the returned message, and
+  // hence must outlive the DynamicMessageFactory.
   //
   // The method is thread-safe.
-  const Message* GetPrototype(const Descriptor* type) override;
+  const Message* PROTOBUF_NONNULL
+  GetPrototype(const Descriptor* PROTOBUF_NONNULL type) override;
 
  private:
-  const DescriptorPool* pool_;
+  const DescriptorPool* PROTOBUF_NULLABLE pool_;
   bool delegate_to_generated_factory_;
 
   struct TypeInfo;
@@ -140,15 +124,17 @@ class PROTOBUF_EXPORT DynamicMessageFactory : public MessageFactory {
   mutable absl::Mutex prototypes_mutex_;
 
   friend class DynamicMessage;
-  const Message* GetPrototypeNoLock(const Descriptor* type);
+  const Message* PROTOBUF_NONNULL
+  GetPrototypeNoLock(const Descriptor* PROTOBUF_NONNULL type);
 };
 
 // Helper for computing a sorted list of map entries via reflection.
 class PROTOBUF_EXPORT DynamicMapSorter {
  public:
-  static std::vector<const Message*> Sort(const Message& message, int map_size,
-                                          const Reflection* reflection,
-                                          const FieldDescriptor* field) {
+  static std::vector<const Message*> Sort(
+      const Message& message, int map_size,
+      const Reflection* PROTOBUF_NONNULL reflection,
+      const FieldDescriptor* PROTOBUF_NONNULL field) {
     std::vector<const Message*> result;
     result.reserve(map_size);
     RepeatedFieldRef<Message> map_field =
@@ -160,7 +146,7 @@ class PROTOBUF_EXPORT DynamicMapSorter {
     std::stable_sort(result.begin(), result.end(), comparator);
     // Complain if the keys aren't in ascending order.
 #ifndef NDEBUG
-    for (size_t j = 1; j < static_cast<size_t>(map_size); j++) {
+    for (size_t j = 1; j < static_cast<size_t>(map_size); ++j) {
       if (!comparator(result[j - 1], result[j])) {
         ABSL_LOG(ERROR) << (comparator(result[j], result[j - 1])
                                 ? "internal error in map key sorting"
@@ -174,10 +160,12 @@ class PROTOBUF_EXPORT DynamicMapSorter {
  private:
   class PROTOBUF_EXPORT MapEntryMessageComparator {
    public:
-    explicit MapEntryMessageComparator(const Descriptor* descriptor)
+    explicit MapEntryMessageComparator(
+        const Descriptor* PROTOBUF_NONNULL descriptor)
         : field_(descriptor->field(0)) {}
 
-    bool operator()(const Message* a, const Message* b) {
+    bool operator()(const Message* PROTOBUF_NONNULL a,
+                    const Message* PROTOBUF_NONNULL b) {
       const Reflection* reflection = a->GetReflection();
       switch (field_->cpp_type()) {
         case FieldDescriptor::CPPTYPE_BOOL: {
@@ -217,7 +205,7 @@ class PROTOBUF_EXPORT DynamicMapSorter {
     }
 
    private:
-    const FieldDescriptor* field_;
+    const FieldDescriptor* PROTOBUF_NONNULL field_;
   };
 };
 
