@@ -32,9 +32,11 @@
 #include "google/protobuf/descriptor.h"
 #include "google/protobuf/io/coded_stream.h"
 #include "google/protobuf/io/zero_copy_stream_impl.h"
+#include "google/protobuf/port.h"
 #include "google/protobuf/test_util.h"
 #include "google/protobuf/text_format.h"
 #include "google/protobuf/unittest.pb.h"
+#include "google/protobuf/unittest_import.pb.h"
 #include "google/protobuf/unittest_lite.pb.h"
 #include "google/protobuf/wire_format.h"
 
@@ -352,7 +354,7 @@ TEST_F(UnknownFieldSetTest, SerializeViaReflection) {
 TEST_F(UnknownFieldSetTest, CopyFrom) {
   unittest::TestEmptyMessage message;
 
-  message.CopyFrom(empty_message_);
+  message = empty_message_;
 
   EXPECT_EQ(empty_message_.DebugString(), message.DebugString());
 }
@@ -644,9 +646,8 @@ TEST_F(UnknownFieldSetTest, SpaceUsed) {
     result += shadow_vector.SpaceUsedExcludingSelfLong();
     result += shadow_vector_group.SpaceUsedExcludingSelfLong();
     if (str != nullptr) {
-      result += sizeof(std::string);
-      static const size_t sso_capacity = std::string().capacity();
-      if (str->capacity() > sso_capacity) result += str->capacity();
+      result += sizeof(std::string) +
+                internal::StringSpaceUsedExcludingSelfLong(*str);
     }
     if (group != nullptr) {
       result += sizeof(UnknownFieldSet);
